#include "totvs.ch"
#include "restful.ch"
#include "fwmvcdef.ch"

/*/{Protheus.doc} GSFAT081
Webservice para integração com o Fluig.
@author Everton Alves
@since 11/03/2020
@type class
/*/
WSRESTFUL GSFAT081 DESCRIPTION "Integração Fluig / Protheus" FORMAT "application/json,text/html"
	Data filial As Character
	Data numero As Character
	Data tipo As Character
	Data aprovador As Character

	WSMETHOD PUT AprovaPedidoCompra ;
		DESCRIPTION "Aprova um pedido de compra." ;
		PATH "/compras/pedidos/v1/aprovacao/{filial}/{numero}/{aprovador}/{tipo}" ;
		WSSYNTAX "/GSFAT081/compras/pedidos/v1/aprovacao/{filial}/{numero}/{aprovador}/{tipo}" ;
		PRODUCES APPLICATION_JSON

END WSRESTFUL

/*/{Protheus.doc} PUT PedidoDeCompra
Aprova um pedido de compra.
@author Everton Alves
@since 01/10/2021
@type method
/*/
WSMETHOD PUT AprovaPedidoCompra PATHPARAM filial, numero, aprovador, tipo WSSERVICE GSFAT081
	Local aSCRArea As Character
	Local aSAKArea As Character
	Local aErro As Character  // Recebe msg de erro de processamento
	Local cFilPed As Character
	Local cNumPed As Character  // Recebe o número do documento a ser avaliado
	Local cTipo As Character  // Recebe o tipo do documento a ser avaliado
	Local cAprov As Character // Recebe o código do aprovador do documento
	Local cSvFilAnt As Character
	Local cSvEmpAnt As Character
	Local cSvArqTab As Character
	Local cDirName As Character
	Local cArquivo As Character
	Local cErro As Character
	Local cMsg As Character
	Local jPedido As Json
	Local lOk As Logical  // Controle de validação e commit
	Local oModel094 As Object  // Objeto que receberá o modelo da MATA094

	Default ::filial := ""
	Default ::numero := ""
	Default ::tipo := "PC"
	Default ::aprovador := ""

	::SetContentType("application/json")

	cSvFilAnt := cFilAnt  // Salva a Filial Anterior
	cSvEmpAnt := cEmpAnt  // Salva a Empresa Anterior
	cSvArqTab := cArqTab  // Salva os arquivos de trabalho

	cFilPed := PadR(::filial, TamSX3("CR_FILIAL")[1])
	cNumPed := PadR(::numero, TamSX3("CR_NUM")[1])
	cTipo := PadR(::tipo, TamSX3("CR_TIPO")[1])
	cAprov := PadR(::aprovador, TamSX3("CR_APROV")[1])

	QOut(">>>> Filial: " + cFilPed)
	QOut(">>>> Pedido: " + cNumPed)
	QOut(">>>> Tipo: " + cTipo)
	QOut(">>>> Aprovador: " + cAprov)

	// Altera filial corrente
	If (cFilAnt <> cFilPed)
		cFilAnt := cFilPed
	EndIf

	DbSelectArea("SCR")  // Documentos com Alçada
	aSCRArea := SCR->(GetArea())
	SCR->(DbSetOrder(3))  // CR_FILIAL+CR_TIPO+CR_NUM+CR_APROV

	DbSelectArea("SAK")  // Aprovadores
	aSAKArea := SAK->(GetArea())
	SAK->(DbSetOrder(1))  // AK_FILIAL+AK_COD

	//Setando __cUserId por conta de uma chamada da função RetCodUsr() no MATA094
	If SAK->(MsSeek(xFilial("SAK") + cAprov))
		//__cUserId Variavel publica
		__cUserId := SAK->AK_USER
	EndIf

	QOut(">>>> Buscando alçada")

	If SCR->(MsSeek(xFilial("SCR") + cTipo + PadR(cNumPed, TamSX3("CR_NUM")[1]) + cAprov))
		QOut(">>>> Alçada encontrada")

		// Códigos de operações possí­veis:
		//    "001" // Liberado
		//    "002" // Estornar
		//    "003" // Superior
		//    "004" // Transferir Superior
		//    "005" // Rejeitado
		//    "006" // Bloqueio
		//    "007" // Visualizacao

		// Seleciona a operação de aprovação de documentos
		A094SetOp("001")

		QOut(">>>> Carregando modelo de dados")

		// Carrega o modelo de dados e seleciona a operação de aprovação (UPDATE)
		oModel094 := FWLoadModel("MATA094")
		oModel094:SetOperation(MODEL_OPERATION_UPDATE)
		oModel094:Activate()

		QOut(">>>> Validando formulário")

		// Valida o formulário
		lOk := oModel094:VldData()

		If lOk
			QOut(">>>> Modelo válido")
			QOut(">>>> Gravando formulário")
			// Se validou, grava o formulário
			lOk := oModel094:CommitData()
		Else
			QOut(">>>> Erro na validação do formulário")
		EndIf

		// Avalia erros
		If lOk
			QOut(">>>> Pedido aprovado")
			QOut(">>>> Pedido posicionado: " + SCR->CR_NUM)

			jPedido := JsonObject():New()
			jPedido["filial"] := SCR->CR_FILIAL
			jPedido["emissao"] := SCR->CR_EMISSAO
			jPedido["pedido"] := AllTrim(SCR->CR_NUM)
			jPedido["tipo"] := SCR->CR_TIPO

			jPedido["aprovador"] := JsonObject():New()
			jPedido["aprovador"]["codigo"] := SCR->CR_LIBAPRO
			jPedido["aprovador"]["nome"] := AllTrim(SAK->AK_NOME)

			jPedido["status"] := JsonObject():New()
			jPedido["status"]["codigo"] := SCR->CR_STATUS

			If (SCR->CR_STATUS == "01") // 01 - Pendente em níveis anteriores
				jPedido["status"]["descricao"] := "Pendente em níveis anteriores"
			ElseIf (SCR->CR_STATUS == "02") // 02 - Pendente
				jPedido["status"]["descricao"] := "Pendente"
			ElseIf (SCR->CR_STATUS == "03") // 03 - Aprovado
				jPedido["status"]["descricao"] := "Aprovado"
			ElseIf (SCR->CR_STATUS == "04") // 04 - Bloqueado
				jPedido["status"]["descricao"] := "Bloqueado"
			ElseIf (SCR->CR_STATUS == "05") // 05 - Aprovado/rejeitado pelo nível
				jPedido["status"]["descricao"] := "Aprovado/rejeitado pelo nível"
			ElseIf (SCR->CR_STATUS == "06") // 06 - Rejeitado
				jPedido["status"]["descricao"] := "Rejeitado"
			EndIf

			jPedido["dataAprovacao"] := SCR->CR_DATALIB
			jPedido["valor"] := SCR->CR_TOTAL
			jPedido["valorLiberado"] := SCR->CR_VALLIB

		Else
			// Busca o Erro do Modelo de Dados
			aErro := oModel094:GetErrorMessage()

			QOut(">>>> Erro na aprovação do pedido.")

			// Monta o Texto que será mostrado na tela
			AutoGrLog("Id do formulário de origem: [" + AllToChar(aErro[01]) + "]")
			AutoGrLog("Id do campo de origem: [" + AllToChar(aErro[02]) + "]")
			AutoGrLog("Id do formulário de erro: [" + AllToChar(aErro[03]) + "]")
			AutoGrLog("Id do campo de erro: [" + AllToChar(aErro[04]) + "]")
			AutoGrLog("Id do erro: [" + AllToChar(aErro[05]) + "]")
			AutoGrLog("Mensagem do erro: [" + AllToChar(aErro[06]) + "]")
			AutoGrLog("Mensagem da solução [" + AllToChar(aErro[07]) + "]")
			AutoGrLog("Valor atribuído: [" + AllToChar(aErro[08]) + "]")
			AutoGrLog("Valor anterior: [" + AllToChar(aErro[09]) + "]")

			cDirName := GetSrvProfString("Startpath", "")
			cArquivo := "ERRO-PC-" + DToS(dDataBase) + StrTran(Time(), ":") + ".TXT"
			MostraErro(cDirName, cArquivo)
			cErro := MemoRead(cArquivo)
			FErase(cDirName + cArquivo)
			DisarmTransaction()

			BeginContent Var cMsg
			<html>
			<body>
			<h2>Aprovação de Pedido</h2>
			<p>Segue abaixo o erro ocorrido na aprovação de pedido de compra:</p>
			<h2>Erro Protheus:</h2>
			<pre>%Exp:cErro%</pre>
			</body>
			</html>
			EndContent

			EnvMail("Aprovação de Pedido", cMsg)
			jPedido := RetCodErro("E026")

			// Mostra a mensagem de Erro
			MostraErro()
		EndIf

		// Desativa o modelo de dados
		oModel094:DeActivate()
	Else
		QOut(">>>> Alçada não encontrada")
		jPedido := RetCodErro("E025")
	EndIf

	::SetResponse(EncodeUtf8(jPedido:toJSON(), "cp1252"))

	If !Empty(jPedido["code"])
		SetRestFault(500, jPedido["message"])
	EndIf

	RestArea(aSCRArea)
	RestArea(aSAKArea)

	// Restaura os Dados de Entrada ( Ambiente )
	cFilAnt := cSvFilAnt
	cEmpAnt := cSvEmpAnt
	cArqTab := cSvArqTab

Return (.T.)


/*/{Protheus.doc} RetErros
Retorna um array com os códigos de erro.
@author Everton Alves
@since 18/09/2020
@return Array, Array com os código de erro
@type function
/*/
Static Function RetErros()
	Local aErros As Array

	aErros := {}
	AAdd(aErros, {"E001", "Ordem de produção não encontrada.", "Não foi encontrada uma ordem de produção com os parâmetros informados."})
	AAdd(aErros, {"E002", "Apontamento não encontrado.", "Não foi encontrada um apontamento de produção com os parâmetros informados."})
	AAdd(aErros, {"E003", "Não foi possível incluir a Ordem de Produção.", "Erro ao incluir a ordem de produção."})
	AAdd(aErros, {"E004", "Parâmetros insuficientes.", "Verifique os parâmetros informados"})
	AAdd(aErros, {"E005", "Quantidade Inválida", "Quantidade informada deve ser maior que zero."})
	AAdd(aErros, {"E006", "Produto não encontrado.", "Verifique o código do produto informado"})
	AAdd(aErros, {"E007", "Não foi possível excluir a Ordem de Produção.", "Verifique se a ordem de produção pode ser excluída."})
	AAdd(aErros, {"E008", "Não foi possível alterar a Ordem de Produção.", "Verifique se a ordem de produção pode ser alterada."})
	AAdd(aErros, {"E009", "Não foi possível apontar a produção.", "Erro ao apontar a ordem de produção."})
	AAdd(aErros, {"E010", "Apontamento já extornado anteriormente.", "Informe um apontamento válido."})
	AAdd(aErros, {"E011", "Código não encontrado.", "Verifique o parâmetro enviado."})
	AAdd(aErros, {"E012", "Arquivos ainda não foram gerados.", "Os arquivos ainda não estão prontos."})
	AAdd(aErros, {"E013", "Pedido de venda não encontrado", "Não foi encontrado um pedido de venda com os parâmetros informados."})
	AAdd(aErros, {"E014", "Erro ao incluir o pedido", "Não foi possível incluir o pedido de venda."})
	AAdd(aErros, {"E015", "Erro ao alterar o pedido", "Não foi possível alterar o pedido de venda."})
	AAdd(aErros, {"E016", "Erro ao excluir o pedido", "Não foi possível excluir o pedido de venda."})
	AAdd(aErros, {"E017", "Cliente não encontrado", "Verifique o código do cliente informado."})
	AAdd(aErros, {"E018", "Pedido de venda já cadastrado", "O pedido de venda já foi cadastrado anteriormente."})
	AAdd(aErros, {"E019", "Nota fiscal não encontrada", "Não foi encontrada uma nota fiscal com os parâmetros informados."})
	AAdd(aErros, {"E020", "Erro ao excluir nota fiscal", "Não foi possível excluir a nota fiscal."})
	AAdd(aErros, {"E021", "Processo do fluig não encontrado", "O código do processo do fluig não foi encontrado no protheus."})
	AAdd(aErros, {"E022", "Movimento não encontrado.", "Não foi encontrado uma movimentação com os parâmetros informados."})
	AAdd(aErros, {"E023", "Erro ao incluir movimentação.", "Não foi possível realizar a movimentação, consulte os logs para detalhes."})
	AAdd(aErros, {"E024", "Erro ao estornar movimentação.", "Não foi possível estornar a movimentação, consulte os logs para detalhes."})
	AAdd(aErros, {"E025", "Objeto não encontrado", "Não foi possível encontrar um objeto com o código informado."})

Return (aErros)

/*/{Protheus.doc} RetCodErro
Retorna um objeto Json com o código de erro.
@author Everton Alves
@since 18/09/2020
@param cErro, Character, Código do erro
@return Object, Objeto Json com o código de erro.
@type function
/*/
Static Function RetCodErro(cErro As Character)
	Local aErros As Array
	Local nPos As Numeric
	Local jErro As Object

	aErros := RetErros()
	jErro := JsonObject():New()
	nPos := Ascan(aErros, {|aTemp| aTemp[1] == cErro})

	If (nPos > 0)
		jErro["code"] := aErros[nPos, 1]
		jErro["message"] := aErros[nPos, 2]
		jErro["detailedMessage"] := aErros[nPos, 3]
	Else
		jErro["code"] := "E999"
		jErro["message"] := "Erro não catalogado."
		jErro["detailedMessage"] := "Não foi possível definir o erro."
	EndIf

Return (jErro)

/*/{Protheus.doc} EnvMail
Envia mensagens de erro por e-mail
@author Everton Alves
@since 19/11/2020
@param cTitulo, Character, Título da mensagem
@param cMsg, Character, Mensagem que será enviada
@type function
/*/
Static Function EnvMail(cTitulo As Character, cMsg As Character)
	Local aEmail As Array
	Local cEmail as Character
	
	cEmail := SuperGetMv("MV_MAILERR", .F., "")

	aEmail := {cEmail}
	U_GSGEN001(cTitulo, cMsg, aEmail)

Return NIL